import { fromHtml } from "hast-util-from-html";
import { toText } from "hast-util-to-text";
import { removePosition } from "unist-util-remove-position";
import { visitParents } from "unist-util-visit-parents";
const languagePattern = /\blanguage-(\S+)\b/;
function highlightCodeBlocks(tree, highlighter) {
  visitParents(tree, { type: "element", tagName: "code" }, (node, ancestors) => {
    const parent = ancestors.at(-1);
    if (parent?.type !== "element" || parent.tagName !== "pre") {
      return;
    }
    if (parent.children.length !== 1) {
      return;
    }
    let languageMatch;
    let { className } = node.properties;
    if (typeof className === "string") {
      languageMatch = className.match(languagePattern);
    } else if (Array.isArray(className)) {
      for (const cls of className) {
        if (typeof cls !== "string") {
          continue;
        }
        languageMatch = cls.match(languagePattern);
        if (languageMatch) {
          break;
        }
      }
    }
    if (languageMatch?.[1] === "math") {
      return;
    }
    const code = toText(node, { whitespace: "pre" });
    const html = highlighter(code, languageMatch?.[1] || "plaintext");
    const replacement = fromHtml(html, { fragment: true }).children[0];
    removePosition(replacement);
    const grandParent = ancestors.at(-2);
    const index = grandParent.children.indexOf(parent);
    grandParent.children[index] = replacement;
  });
}
export {
  highlightCodeBlocks
};
